@@ -39,12 +39,58 @@
 #include "i965_encoder.h"
 #include "gen6_vme.h"
 #include "gen6_mfc.h"
-#include "gen9_mfc.h"
 
-extern Bool gen6_mfc_context_init(VADriverContextP ctx, struct intel_encoder_context *encoder_context);
-extern Bool gen6_vme_context_init(VADriverContextP ctx, struct intel_encoder_context *encoder_context);
-extern Bool gen7_mfc_context_init(VADriverContextP ctx, struct intel_encoder_context *encoder_context);
-extern Bool gen9_hcpe_context_init(VADriverContextP ctx, struct intel_encoder_context *encoder_context);
+static VAStatus
+clear_border(struct object_surface *obj_surface)
+{
+    int width[3], height[3], hstride[3], vstride[3]; /* in byte */
+    int planes;
+    unsigned char* p;
+    int i,j;
+
+    if (obj_surface->border_cleared)
+        return VA_STATUS_SUCCESS;
+
+    if (obj_surface->fourcc == VA_FOURCC_NV12) {
+        planes = 2;
+        width[0] = width[1] = obj_surface->orig_width;
+        height[0] = obj_surface->orig_height;
+        height[1] = obj_surface->orig_height / 2;
+        hstride[0] = hstride[1] = obj_surface->width;
+        vstride[0]= obj_surface->height;
+        vstride[1] = obj_surface->height / 2;
+
+    } else {
+        /* todo add P010 */
+        return VA_STATUS_SUCCESS;
+    }
+    drm_intel_gem_bo_map_gtt(obj_surface->bo);
+
+    p = (unsigned char*)obj_surface->bo->virtual;
+    if (!p)
+        return VA_STATUS_ERROR_INVALID_SURFACE;
+
+    for (i = 0; i < planes; i++) {
+        int w = width[i];
+        int h = height[i];
+        int hs = hstride[i];
+        int vs = vstride[i];
+        /* right */
+        for (j = 0; j < h; j++) {
+            memset(p + w, 0, hs - w);
+            p += hs;
+        }
+        /* bottom */
+        for (/* nothing */; j < vs; j++) {
+            memset(p, 0, hs);
+            p += hs;
+        }
+
+    }
+    drm_intel_gem_bo_unmap_gtt(obj_surface->bo);
+    obj_surface->border_cleared = true;
+    return VA_STATUS_SUCCESS;
+}
 
 static VAStatus
 intel_encoder_check_yuv_surface(VADriverContextP ctx,
@@ -71,15 +117,21 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
     if (!obj_surface || !obj_surface->bo)
         return VA_STATUS_ERROR_INVALID_PARAMETER;
 
-    if (obj_surface->fourcc == VA_FOURCC_NV12) {
-        unsigned int tiling = 0, swizzle = 0;
+    if (VAProfileHEVCMain10 == profile &&
+        obj_surface->fourcc != VA_FOURCC_P010)
+        return VA_STATUS_ERROR_INVALID_PARAMETER;
 
+    if (obj_surface->fourcc == VA_FOURCC_NV12 ||
+        (VAProfileHEVCMain10 == profile &&
+        obj_surface->fourcc == VA_FOURCC_P010)) {
+
+        unsigned int tiling = 0, swizzle = 0;
         dri_bo_get_tiling(obj_surface->bo, &tiling, &swizzle);
 
         if (tiling == I915_TILING_Y) {
             encoder_context->input_yuv_surface = encode_state->current_render_target;
             encode_state->input_yuv_object = obj_surface;
-            return VA_STATUS_SUCCESS;
+            return clear_border(obj_surface);
         }
     }
 
@@ -98,10 +150,7 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
                                  VA_RT_FORMAT_YUV420,
                                  1,
                                  &encoder_context->input_yuv_surface);
-    assert(status == VA_STATUS_SUCCESS);
-
-    if (status != VA_STATUS_SUCCESS)
-        return status;
+    ASSERT_RET(status == VA_STATUS_SUCCESS, status);
 
     obj_surface = SURFACE(encoder_context->input_yuv_surface);
     encode_state->input_yuv_object = obj_surface;
@@ -121,7 +170,7 @@ intel_encoder_check_yuv_surface(VADriverContextP ctx,
 
     encoder_context->is_tmp_id = 1;
 
-    return VA_STATUS_SUCCESS;
+    return clear_border(obj_surface);
 }
 
 
@@ -250,22 +299,65 @@ intel_encoder_check_jpeg_yuv_surface(VADriverContextP ctx,
 }
 
 static VAStatus
-intel_encoder_check_misc_parameter(VADriverContextP ctx,
-                                  struct encode_state *encode_state,
-                                  struct intel_encoder_context *encoder_context)
+intel_encoder_check_brc_h264_sequence_parameter(VADriverContextP ctx,
+                                                struct encode_state *encode_state,
+                                                struct intel_encoder_context *encoder_context)
 {
+    VAEncSequenceParameterBufferH264 *seq_param = (VAEncSequenceParameterBufferH264 *)encode_state->seq_param_ext->buffer;
+    unsigned short num_pframes_in_gop, num_bframes_in_gop;
+    unsigned int bits_per_second, framerate_per_100s;
 
-    if (encode_state->misc_param[VAEncMiscParameterTypeQualityLevel] &&
-        encode_state->misc_param[VAEncMiscParameterTypeQualityLevel]->buffer) {
-        VAEncMiscParameterBuffer* pMiscParam = (VAEncMiscParameterBuffer*)encode_state->misc_param[VAEncMiscParameterTypeQualityLevel]->buffer;
-        VAEncMiscParameterBufferQualityLevel* param_quality_level = (VAEncMiscParameterBufferQualityLevel*)pMiscParam->data;
-        encoder_context->quality_level = param_quality_level->quality_level;
+    if (!encoder_context->is_new_sequence)
+        return VA_STATUS_SUCCESS;
 
-        if (encoder_context->quality_level == 0)
-            encoder_context->quality_level = ENCODER_DEFAULT_QUALITY;
-        else if (encoder_context->quality_level > encoder_context->quality_range)
+    assert(seq_param);
+    bits_per_second = seq_param->bits_per_second; // for the highest layer
+
+    if (!seq_param->num_units_in_tick || !seq_param->time_scale)
+        framerate_per_100s = 3000;
+    else
+        framerate_per_100s = seq_param->time_scale * 100 / (2 * seq_param->num_units_in_tick); // for the highest layer
+
+    encoder_context->brc.num_iframes_in_gop = 1; // Always 1
+
+    if (seq_param->intra_period == 0) { // E.g. IDRPP... / IDR(PBB)... (no IDR/I any more)
+        if (seq_param->ip_period == 0)
             goto error;
-   }
+
+        encoder_context->brc.gop_size = (unsigned int)(framerate_per_100s / 100.0 + 0.5); // fake
+        num_pframes_in_gop = (encoder_context->brc.gop_size +
+                              seq_param->ip_period - 1) / seq_param->ip_period - 1;
+    } else if (seq_param->intra_period == 1) { // E.g. IDRIII...
+        encoder_context->brc.gop_size = 1;
+        num_pframes_in_gop = 0;
+    } else {
+        if (seq_param->ip_period == 0)
+            goto error;
+
+        encoder_context->brc.gop_size = seq_param->intra_period;
+        num_pframes_in_gop = (encoder_context->brc.gop_size +
+                              seq_param->ip_period - 1) / seq_param->ip_period - 1;
+    }
+
+    num_bframes_in_gop = (encoder_context->brc.gop_size -
+                          encoder_context->brc.num_iframes_in_gop - num_pframes_in_gop);
+
+    if (num_pframes_in_gop != encoder_context->brc.num_pframes_in_gop ||
+        num_bframes_in_gop != encoder_context->brc.num_bframes_in_gop ||
+        bits_per_second != encoder_context->brc.bits_per_second[encoder_context->layer.num_layers - 1] ||
+        framerate_per_100s != encoder_context->brc.framerate_per_100s[encoder_context->layer.num_layers - 1]) {
+        encoder_context->brc.num_pframes_in_gop = num_pframes_in_gop;
+        encoder_context->brc.num_bframes_in_gop = num_bframes_in_gop;
+        encoder_context->brc.bits_per_second[encoder_context->layer.num_layers - 1] = bits_per_second;
+        encoder_context->brc.framerate_per_100s[encoder_context->layer.num_layers - 1] = framerate_per_100s;
+        encoder_context->brc.need_reset = 1;
+    }
+
+    if (!encoder_context->brc.hrd_buffer_size ||
+        !encoder_context->brc.hrd_initial_buffer_fullness) {
+        encoder_context->brc.hrd_buffer_size = seq_param->bits_per_second << 1;
+        encoder_context->brc.hrd_initial_buffer_fullness = seq_param->bits_per_second;
+    }
 
     return VA_STATUS_SUCCESS;
 
@@ -273,6 +365,227 @@ error:
     return VA_STATUS_ERROR_INVALID_PARAMETER;
 }
 
+static VAStatus
+intel_encoder_check_brc_sequence_parameter(VADriverContextP ctx,
+                                           struct encode_state *encode_state,
+                                           struct intel_encoder_context *encoder_context)
+{
+    if (encoder_context->codec == CODEC_H264 ||
+        encoder_context->codec == CODEC_H264_MVC)
+        return intel_encoder_check_brc_h264_sequence_parameter(ctx, encode_state, encoder_context);
+
+    // TODO: other codecs
+    return VA_STATUS_SUCCESS;
+}
+
+static void
+intel_encoder_check_rate_control_parameter(VADriverContextP ctx,
+                                           struct intel_encoder_context *encoder_context,
+                                           VAEncMiscParameterRateControl *misc)
+{
+    int temporal_id = 0;
+
+    if (encoder_context->layer.num_layers >= 2)
+        temporal_id = misc->rc_flags.bits.temporal_id;
+
+    if (temporal_id >= encoder_context->layer.num_layers)
+        return;
+
+    // TODO: for VBR
+    if (encoder_context->brc.bits_per_second[temporal_id] != misc->bits_per_second) {
+        encoder_context->brc.bits_per_second[temporal_id] = misc->bits_per_second;
+        encoder_context->brc.need_reset = 1;
+    }
+}
+
+static void
+intel_encoder_check_hrd_parameter(VADriverContextP ctx,
+                                  struct intel_encoder_context *encoder_context,
+                                  VAEncMiscParameterHRD *misc)
+{
+    if (encoder_context->brc.hrd_buffer_size != misc->buffer_size ||
+        encoder_context->brc.hrd_initial_buffer_fullness != misc->initial_buffer_fullness) {
+        encoder_context->brc.hrd_buffer_size = misc->buffer_size;
+        encoder_context->brc.hrd_initial_buffer_fullness = misc->initial_buffer_fullness;
+        encoder_context->brc.need_reset = 1;
+    }
+}
+
+static void
+intel_encoder_check_framerate_parameter(VADriverContextP ctx,
+                                        struct intel_encoder_context *encoder_context,
+                                        VAEncMiscParameterFrameRate *misc)
+{
+    int framerate_per_100s;
+    int temporal_id = 0;
+
+    if (encoder_context->layer.num_layers >= 2)
+        temporal_id = misc->framerate_flags.bits.temporal_id;
+
+    if (temporal_id >= encoder_context->layer.num_layers)
+        return;
+
+    if (misc->framerate & 0xffff0000)
+        framerate_per_100s = (misc->framerate & 0xffff) * 100 / ((misc->framerate >> 16) & 0xffff);
+    else
+        framerate_per_100s = misc->framerate * 100;
+
+    if (encoder_context->brc.framerate_per_100s[temporal_id] != framerate_per_100s) {
+        encoder_context->brc.framerate_per_100s[temporal_id] = framerate_per_100s;
+        encoder_context->brc.need_reset = 1;
+    }
+}
+
+static VAStatus
+intel_encoder_check_brc_parameter(VADriverContextP ctx,
+                                  struct encode_state *encode_state,
+                                  struct intel_encoder_context *encoder_context)
+{
+    VAStatus ret;
+    VAEncMiscParameterBuffer *misc_param;
+    int i, j;
+
+    if (!(encoder_context->rate_control_mode & (VA_RC_CBR | VA_RC_VBR)))
+        return VA_STATUS_SUCCESS;
+
+    ret = intel_encoder_check_brc_sequence_parameter(ctx, encode_state, encoder_context);
+
+    if (ret)
+        return ret;
+
+    for (i = 0; i < ARRAY_ELEMS(encode_state->misc_param); i++) {
+        for (j = 0; j < ARRAY_ELEMS(encode_state->misc_param[0]); j++) {
+            if (!encode_state->misc_param[i][j] || !encode_state->misc_param[i][j]->buffer)
+                continue;
+
+            misc_param = (VAEncMiscParameterBuffer *)encode_state->misc_param[i][j]->buffer;
+
+            switch (misc_param->type) {
+            case VAEncMiscParameterTypeFrameRate:
+                intel_encoder_check_framerate_parameter(ctx,
+                                                        encoder_context,
+                                                        (VAEncMiscParameterFrameRate *)misc_param->data);
+                break;
+
+            case VAEncMiscParameterTypeRateControl:
+                intel_encoder_check_rate_control_parameter(ctx,
+                                                           encoder_context,
+                                                           (VAEncMiscParameterRateControl *)misc_param->data);
+                break;
+
+            case VAEncMiscParameterTypeHRD:
+                intel_encoder_check_hrd_parameter(ctx,
+                                                  encoder_context,
+                                                  (VAEncMiscParameterHRD *)misc_param->data);
+                break;
+
+            default:
+                break;
+            }
+        }
+    }
+
+    return VA_STATUS_SUCCESS;
+}
+
+static VAStatus
+intel_encoder_check_temporal_layer_structure(VADriverContextP ctx,
+                                             struct encode_state *encode_state,
+                                             struct intel_encoder_context *encoder_context)
+{
+    VAEncMiscParameterBuffer* misc_param;
+    VAEncMiscParameterTemporalLayerStructure *tls_paramter;
+    unsigned int rate_control_mode = encoder_context->rate_control_mode;
+    int i;
+
+    if (!encoder_context->is_new_sequence) {
+        if (encoder_context->layer.num_layers > 1)
+            encoder_context->layer.curr_frame_layer_id = encoder_context->layer.frame_layer_ids[(encoder_context->num_frames_in_sequence - 1) % encoder_context->layer.size_frame_layer_ids];
+        else
+            encoder_context->layer.curr_frame_layer_id = 0;
+
+        return VA_STATUS_SUCCESS;
+    }
+
+    if (!(rate_control_mode & (VA_RC_CBR | VA_RC_VBR)))
+        return VA_STATUS_SUCCESS;
+
+    if (!encode_state->misc_param[VAEncMiscParameterTypeTemporalLayerStructure][0] ||
+        !encode_state->misc_param[VAEncMiscParameterTypeTemporalLayerStructure][0]->buffer)
+        return VA_STATUS_SUCCESS;
+
+    misc_param = (VAEncMiscParameterBuffer*)encode_state->misc_param[VAEncMiscParameterTypeTemporalLayerStructure][0]->buffer;
+    tls_paramter = (VAEncMiscParameterTemporalLayerStructure *)misc_param->data;
+
+    if (tls_paramter->number_of_layers <= 1)
+        return VA_STATUS_SUCCESS;
+
+    if (tls_paramter->number_of_layers > MAX_TEMPORAL_LAYERS)
+        return VA_STATUS_ERROR_INVALID_PARAMETER;
+
+    if (tls_paramter->periodicity > 32 || tls_paramter->periodicity <= 1)
+        return VA_STATUS_ERROR_INVALID_PARAMETER;
+
+    for (i = 0; i < tls_paramter->number_of_layers; i++) {
+        if (!encode_state->misc_param[VAEncMiscParameterTypeRateControl][i] ||
+            !encode_state->misc_param[VAEncMiscParameterTypeRateControl][i]->buffer ||
+            !encode_state->misc_param[VAEncMiscParameterTypeFrameRate][i] ||
+            !encode_state->misc_param[VAEncMiscParameterTypeFrameRate][i]->buffer) {
+
+            return VA_STATUS_ERROR_INVALID_PARAMETER;
+        }
+    }
+
+    encoder_context->layer.size_frame_layer_ids = tls_paramter->periodicity;
+    encoder_context->layer.num_layers = tls_paramter->number_of_layers;
+
+    for (i = 0; i < encoder_context->layer.size_frame_layer_ids; i++) {
+        if (tls_paramter->layer_id[i] >= tls_paramter->number_of_layers)
+            return VA_STATUS_ERROR_INVALID_PARAMETER;
+
+        encoder_context->layer.frame_layer_ids[i] = tls_paramter->layer_id[i];
+    }
+
+    if (encoder_context->is_new_sequence)
+        encoder_context->layer.curr_frame_layer_id = 0;
+    else
+        encoder_context->layer.curr_frame_layer_id = encoder_context->layer.frame_layer_ids[(encoder_context->num_frames_in_sequence - 1) % encoder_context->layer.size_frame_layer_ids];
+
+    return VA_STATUS_SUCCESS;
+}
+
+static VAStatus
+intel_encoder_check_misc_parameter(VADriverContextP ctx,
+                                  struct encode_state *encode_state,
+                                  struct intel_encoder_context *encoder_context)
+{
+    VAStatus ret = VA_STATUS_SUCCESS;
+
+    if (encode_state->misc_param[VAEncMiscParameterTypeQualityLevel][0] &&
+        encode_state->misc_param[VAEncMiscParameterTypeQualityLevel][0]->buffer) {
+        VAEncMiscParameterBuffer* pMiscParam = (VAEncMiscParameterBuffer*)encode_state->misc_param[VAEncMiscParameterTypeQualityLevel][0]->buffer;
+        VAEncMiscParameterBufferQualityLevel* param_quality_level = (VAEncMiscParameterBufferQualityLevel*)pMiscParam->data;
+        encoder_context->quality_level = param_quality_level->quality_level;
+
+        if (encoder_context->quality_level == 0)
+            encoder_context->quality_level = ENCODER_DEFAULT_QUALITY;
+        else if (encoder_context->quality_level > encoder_context->quality_range) {
+            ret = VA_STATUS_ERROR_INVALID_PARAMETER;
+            goto out;
+        }
+    }
+
+    ret = intel_encoder_check_temporal_layer_structure(ctx, encode_state, encoder_context);
+
+    if (ret)
+        goto out;
+
+    ret = intel_encoder_check_brc_parameter(ctx, encode_state, encoder_context);
+
+out:
+    return ret;
+}
+
 static VAStatus
 intel_encoder_check_avc_parameter(VADriverContextP ctx,
                                   struct encode_state *encode_state,
@@ -282,6 +595,7 @@ intel_encoder_check_avc_parameter(VADriverContextP ctx,
     struct object_surface *obj_surface;	
     struct object_buffer *obj_buffer;
     VAEncPictureParameterBufferH264 *pic_param = (VAEncPictureParameterBufferH264 *)encode_state->pic_param_ext->buffer;
+    VAEncSequenceParameterBufferH264 *seq_param = (VAEncSequenceParameterBufferH264 *)encode_state->seq_param_ext->buffer;
     int i;
 
     assert(!(pic_param->CurrPic.flags & VA_PICTURE_H264_INVALID));
@@ -324,7 +638,20 @@ intel_encoder_check_avc_parameter(VADriverContextP ctx,
 
     for ( ; i < 16; i++)
         encode_state->reference_objects[i] = NULL;
-    
+
+    /*
+     * A sequence consists of an IDR unit, followed by zero or more non-IDR unit, but not including any
+     * subsequent IDR unit, so idr_pic_flag can indicate the current frame is the start of a new
+     * sequnce
+     */
+    encoder_context->is_new_sequence = (pic_param->pic_fields.bits.idr_pic_flag && seq_param);
+
+    if (encoder_context->is_new_sequence) {
+        encoder_context->num_frames_in_sequence = 0;
+        encoder_context->frame_width_in_pixel = seq_param->picture_width_in_mbs * 16;
+        encoder_context->frame_height_in_pixel = seq_param->picture_height_in_mbs * 16;
+    }
+
     return VA_STATUS_SUCCESS;
 
 error:
@@ -441,7 +768,7 @@ intel_encoder_check_vp8_parameter(VADriverContextP ctx,
     if (!obj_surface)
         goto error;
     
-    encode_state->reconstructed_object = obj_surface;    
+    encode_state->reconstructed_object = obj_surface;
     obj_buffer = BUFFER(pic_param->coded_buf);
     assert(obj_buffer && obj_buffer->buffer_store && obj_buffer->buffer_store->bo);
 
@@ -557,6 +884,76 @@ intel_encoder_check_hevc_parameter(VADriverContextP ctx,
 error:
     return VA_STATUS_ERROR_INVALID_PARAMETER;
 }
+
+static VAStatus
+intel_encoder_check_vp9_parameter(VADriverContextP ctx,
+                                  struct encode_state *encode_state,
+                                  struct intel_encoder_context *encoder_context)
+{
+    struct i965_driver_data *i965 = i965_driver_data(ctx);
+    VAEncPictureParameterBufferVP9 *pic_param;
+    struct object_surface *obj_surface;
+    struct object_buffer *obj_buffer;
+    int i = 0;
+    int is_key_frame = 0;
+    int index;
+
+    if (encode_state->pic_param_ext == NULL ||
+        encode_state->pic_param_ext->buffer == NULL)
+        return VA_STATUS_ERROR_INVALID_PARAMETER;
+
+    pic_param = (VAEncPictureParameterBufferVP9 *)encode_state->pic_param_ext->buffer;
+
+    obj_surface = SURFACE(pic_param->reconstructed_frame);
+
+    if (!obj_surface)
+        goto error;
+
+    encode_state->reconstructed_object = obj_surface;
+    obj_buffer = BUFFER(pic_param->coded_buf);
+
+    if (!obj_buffer || !obj_buffer->buffer_store || !obj_buffer->buffer_store->bo)
+        goto error;
+
+    encode_state->coded_buf_object = obj_buffer;
+
+    is_key_frame = !pic_param->pic_flags.bits.frame_type;
+    if (!is_key_frame && !pic_param->pic_flags.bits.intra_only) {
+        /* slot 0 is for last reference frame */
+        index = pic_param->ref_flags.bits.ref_last_idx;
+        obj_surface = SURFACE(pic_param->reference_frames[index]);
+        if (obj_surface && obj_surface->bo)
+            encode_state->reference_objects[i++] = obj_surface;
+        else
+            encode_state->reference_objects[i++] = NULL;
+
+        /* slot 1 is for golden reference frame */
+        index = pic_param->ref_flags.bits.ref_gf_idx;
+        obj_surface = SURFACE(pic_param->reference_frames[index]);
+        if (obj_surface && obj_surface->bo)
+            encode_state->reference_objects[i++] = obj_surface;
+        else
+            encode_state->reference_objects[i++] = NULL;
+
+        /* slot 2 is alt reference frame */
+        index = pic_param->ref_flags.bits.ref_arf_idx;
+        obj_surface = SURFACE(pic_param->reference_frames[index]);
+        if (obj_surface && obj_surface->bo)
+            encode_state->reference_objects[i++] = obj_surface;
+        else
+            encode_state->reference_objects[i++] = NULL;
+    }
+
+    for ( ; i < 16; i++)
+        encode_state->reference_objects[i] = NULL;
+
+    return VA_STATUS_SUCCESS;
+
+error:
+    return VA_STATUS_ERROR_INVALID_PARAMETER;
+}
+
+
 static VAStatus
 intel_encoder_sanity_check_input(VADriverContextP ctx,
                                  VAProfile profile,
@@ -603,13 +1000,22 @@ intel_encoder_sanity_check_input(VADriverContextP ctx,
         break;
     }
 
-    case VAProfileHEVCMain:  {
+    case VAProfileHEVCMain:
+    case VAProfileHEVCMain10:  {
         vaStatus = intel_encoder_check_hevc_parameter(ctx, encode_state, encoder_context);
         if (vaStatus != VA_STATUS_SUCCESS)
             goto out;
         vaStatus = intel_encoder_check_yuv_surface(ctx, profile, encode_state, encoder_context);
         break;
     }
+
+    case VAProfileVP9Profile0: {
+        vaStatus = intel_encoder_check_vp9_parameter(ctx, encode_state, encoder_context);
+        if (vaStatus != VA_STATUS_SUCCESS)
+            goto out;
+        vaStatus = intel_encoder_check_yuv_surface(ctx, profile, encode_state, encoder_context);
+        break;
+    }
     default:
         vaStatus = VA_STATUS_ERROR_UNSUPPORTED_PROFILE;
         break;
@@ -641,10 +1047,14 @@ intel_encoder_end_picture(VADriverContextP ctx,
 
     if((encoder_context->vme_context && encoder_context->vme_pipeline)) {
         vaStatus = encoder_context->vme_pipeline(ctx, profile, encode_state, encoder_context);
+        if (vaStatus != VA_STATUS_SUCCESS)
+            return vaStatus;
     }
 
-    if (vaStatus == VA_STATUS_SUCCESS)
-        encoder_context->mfc_pipeline(ctx, profile, encode_state, encoder_context);
+    encoder_context->mfc_pipeline(ctx, profile, encode_state, encoder_context);
+    encoder_context->num_frames_in_sequence++;
+    encoder_context->brc.need_reset = 0;
+
     return VA_STATUS_SUCCESS;
 }
 
@@ -658,10 +1068,28 @@ intel_encoder_context_destroy(void *hw_context)
     if (encoder_context->vme_context_destroy && encoder_context->vme_context)
        encoder_context->vme_context_destroy(encoder_context->vme_context);
 
+    if (encoder_context->enc_priv_state) {
+        free(encoder_context->enc_priv_state);
+        encoder_context->enc_priv_state = NULL;
+    }
+
     intel_batchbuffer_free(encoder_context->base.batch);
     free(encoder_context);
 }
 
+
+static VAStatus
+intel_encoder_get_status(VADriverContextP ctx, struct hw_context *hw_context, void *buffer)
+{
+    struct intel_encoder_context *encoder_context = (struct intel_encoder_context *)hw_context;
+    struct i965_coded_buffer_segment *coded_buffer_segment = (struct i965_coded_buffer_segment *)buffer;
+
+    if (encoder_context->get_status)
+        return encoder_context->get_status(ctx, encoder_context, coded_buffer_segment);
+
+    return VA_STATUS_ERROR_UNIMPLEMENTED;
+}
+
 typedef Bool (* hw_init_func)(VADriverContextP, struct intel_encoder_context *);
 
 static struct hw_context *
@@ -677,12 +1105,18 @@ intel_enc_hw_context_init(VADriverContextP ctx,
     assert(encoder_context);
     encoder_context->base.destroy = intel_encoder_context_destroy;
     encoder_context->base.run = intel_encoder_end_picture;
+    encoder_context->base.get_status = intel_encoder_get_status;
     encoder_context->base.batch = intel_batchbuffer_new(intel, I915_EXEC_RENDER, 0);
     encoder_context->input_yuv_surface = VA_INVALID_SURFACE;
     encoder_context->is_tmp_id = 0;
+    encoder_context->low_power_mode = 0;
     encoder_context->rate_control_mode = VA_RC_NONE;
     encoder_context->quality_level = ENCODER_DEFAULT_QUALITY;
     encoder_context->quality_range = 1;
+    encoder_context->layer.num_layers = 1;
+
+    if (obj_config->entrypoint == VAEntrypointEncSliceLP)
+        encoder_context->low_power_mode = 1;
 
     switch (obj_config->profile) {
     case VAProfileMPEG2Simple:
@@ -694,7 +1128,11 @@ intel_enc_hw_context_init(VADriverContextP ctx,
     case VAProfileH264Main:
     case VAProfileH264High:
         encoder_context->codec = CODEC_H264;
-        encoder_context->quality_range = ENCODER_QUALITY_RANGE;
+
+        if (obj_config->entrypoint == VAEntrypointEncSliceLP)
+            encoder_context->quality_range = ENCODER_LP_QUALITY_RANGE;
+        else
+            encoder_context->quality_range = ENCODER_QUALITY_RANGE;
         break;
 
     case VAProfileH264StereoHigh:
@@ -711,9 +1149,14 @@ intel_enc_hw_context_init(VADriverContextP ctx,
         break;
 
     case VAProfileHEVCMain:
+    case VAProfileHEVCMain10:
         encoder_context->codec = CODEC_HEVC;
         break;
 
+    case VAProfileVP9Profile0:
+        encoder_context->codec = CODEC_VP9;
+        break;
+
     default:
         /* Never get here */
         assert(0);
@@ -729,16 +1172,17 @@ intel_enc_hw_context_init(VADriverContextP ctx,
                 WARN_ONCE("Don't support CBR for MPEG-2 encoding\n");
                 encoder_context->rate_control_mode &= ~VA_RC_CBR;
             }
-
-            break;
+        }
+        if (obj_config->attrib_list[i].type == VAConfigAttribEncROI) {
+            if (encoder_context->codec == CODEC_H264)
+                encoder_context->context_roi = 1;
         }
     }
 
-    vme_context_init(ctx, encoder_context);
-    if(obj_config->profile != VAProfileJPEGBaseline) {
-        assert(encoder_context->vme_context);
-        assert(encoder_context->vme_context_destroy);
-        assert(encoder_context->vme_pipeline);
+    if (vme_context_init) {
+        vme_context_init(ctx, encoder_context);
+        assert(!encoder_context->vme_context ||
+               (encoder_context->vme_context_destroy && encoder_context->vme_pipeline));
     }
 
     mfc_context_init(ctx, encoder_context);
@@ -777,10 +1221,5 @@ gen8_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config)
 struct hw_context *
 gen9_enc_hw_context_init(VADriverContextP ctx, struct object_config *obj_config)
 {
-    if (obj_config->profile == VAProfileHEVCMain) {
-        return intel_enc_hw_context_init(ctx, obj_config, gen9_vme_context_init, gen9_hcpe_context_init);
-    } else if (obj_config->profile == VAProfileJPEGBaseline)
-        return intel_enc_hw_context_init(ctx, obj_config, gen8_vme_context_init, gen8_mfc_context_init);
-    else
-        return intel_enc_hw_context_init(ctx, obj_config, gen9_vme_context_init, gen9_mfc_context_init);
+    return intel_enc_hw_context_init(ctx, obj_config, gen9_vme_context_init, gen9_mfc_context_init);
 }
