@@ -36,17 +36,66 @@
 #include "i965_structs.h"
 #include "i965_drv_video.h"
 
+#define I965_BRC_NONE	                0
+#define I965_BRC_CBR	                1
+#define I965_BRC_VBR	                2
+#define I965_BRC_CQP	                3
+
+#define WIDTH_IN_MACROBLOCKS(width)     (ALIGN(width, 16) >> 4)
+#define HEIGHT_IN_MACROBLOCKS(height)   (ALIGN(height, 16) >> 4)
+#define MAX_TEMPORAL_LAYERS	        4
+
+struct intel_roi
+{
+    short left;
+    short right;
+    short top;
+    short bottom;
+
+    char  value;
+};
+
 struct intel_encoder_context
 {
     struct hw_context base;
     int codec;
     VASurfaceID input_yuv_surface;
-    int is_tmp_id;
     unsigned int rate_control_mode;
     unsigned int quality_level;
     unsigned int quality_range;
+    unsigned int num_frames_in_sequence;
+    unsigned int frame_width_in_pixel;
+    unsigned int frame_height_in_pixel;
+
+    struct {
+        unsigned int num_layers;
+        unsigned int size_frame_layer_ids;
+        unsigned int frame_layer_ids[32];
+        unsigned int curr_frame_layer_id;
+    } layer;
+
+    struct {
+        unsigned short gop_size;
+        unsigned short num_iframes_in_gop;
+        unsigned short num_pframes_in_gop;
+        unsigned short num_bframes_in_gop;
+        unsigned int bits_per_second[MAX_TEMPORAL_LAYERS];
+        unsigned int framerate_per_100s[MAX_TEMPORAL_LAYERS];
+        unsigned int hrd_buffer_size;
+        unsigned int hrd_initial_buffer_fullness;
+        unsigned int need_reset;
+    } brc;
+
     void *vme_context;
     void *mfc_context;
+    void *enc_priv_state;
+
+    unsigned int is_tmp_id:1;
+    unsigned int low_power_mode:1;
+    unsigned int soft_batch_force:1;
+    unsigned int context_roi:1;
+    unsigned int is_new_sequence:1; /* Currently only valid for H.264, TODO for other codecs */
+
     void (*vme_context_destroy)(void *vme_context);
     VAStatus (*vme_pipeline)(VADriverContextP ctx,
                              VAProfile profile,
@@ -59,6 +108,10 @@ struct intel_encoder_context
                              struct intel_encoder_context *encoder_context);
     void (*mfc_brc_prepare)(struct encode_state *encode_state,
                             struct intel_encoder_context *encoder_context);
+
+    VAStatus (*get_status)(VADriverContextP ctx,
+                           struct intel_encoder_context *encoder_context,
+                           struct i965_coded_buffer_segment *coded_buffer_segment);
 };
 
 extern struct hw_context *
