@@ -57,6 +57,11 @@ struct gen_hevc_surface
 {
     GenCodecSurface base;
     dri_bo *motion_vector_temporal_bo;
+    //Encoding HEVC10:internal surface keep for P010->NV12 , this is only for hevc10 to save the P010->NV12
+    struct object_surface *nv12_surface_obj;
+    VASurfaceID nv12_surface_id;
+    VADriverContextP ctx;
+    int has_p010_to_nv12_done;
 };
 
 typedef struct gen_vp9_surface GenVP9Surface;
@@ -68,8 +73,18 @@ struct gen_vp9_surface
     dri_bo *motion_vector_temporal_bo;
 };
 
+typedef struct vdenc_avc_surface VDEncAvcSurface;
+struct vdenc_avc_surface
+{
+    VADriverContextP ctx;
+    VASurfaceID scaled_4x_surface_id;
+    struct object_surface *scaled_4x_surface_obj;
+};
+
 extern void gen_free_hevc_surface(void **data);
 
 extern void gen_free_vp9_surface(void **data);
 
+extern void vdenc_free_avc_surface(void **data);
+
 #endif /* INTEL_MEDIA_H */
